# -*- coding: utf-8 -*-

from django.db.models import Count, F

from kinder.core.alloc.helpers import GroupHelper
from kinder.core.dict.models import GroupState
from kinder.core.direct.models import Direct, DRS
from kinder.core.group.models import Pupil
from gisdo.index import BaseIndex
from gisdo.algorithm import (
    CommonCountAlgorithm, ShortStayAlgorithm,
    CapacitiesHealthNeedAlgorithm, CompensatingAlgorithm,
    HealthGroupAlgorithm)
from gisdo.constants import (
    GOVERNMENT, AGE_CUT_SUBCATEGORY_MAP,
    AGE_CUT_CATEGORY_MAP, AGE_EIGHT_SUBCATEGORY_MAP,
    AGE_EIGHT_CATEGORY_MAP, AGE_CATEGORIES_CUT,
    AGE_CUT_DEFAULT_CATEGORY, ALL_UNIT_TYPES,
    AGE_EIGHT_DEFAULT_CATEGORY, DOU_TYPE_MAP)


class CapacitiesIndex(BaseIndex):
    """
    """

    COMMON_COUNT = 1
    SHORT_STAY = 2
    HEALTH_NEED = 3
    COMPENSATING = 4
    HEALTH = 5

    ALGORITHM_MAP = {
        COMMON_COUNT: CommonCountAlgorithm,
        SHORT_STAY: ShortStayAlgorithm,
        HEALTH_NEED: CapacitiesHealthNeedAlgorithm,
        COMPENSATING: CompensatingAlgorithm,
        HEALTH: HealthGroupAlgorithm
    }

    class Ctx(object):

        kind = 1

        def __init__(self, unit_id):

            self.group_unit = unit_id
            self.group_states = list(GroupState.objects.values_list(
                "code", flat=True))

    def __init__(self, unit):

        super(CapacitiesIndex, self).__init__(unit, session=None)

        _, self._cache = GisdoGroupHelper(
            CapacitiesIndex.Ctx(self.unit.id)).getList()

    @staticmethod
    def _get_age_map(age_category):

        cut_values = AGE_CATEGORIES_CUT.values()

        if age_category in cut_values:
            subcategory_map, category_map, default_category = (
                AGE_CUT_SUBCATEGORY_MAP, AGE_CUT_CATEGORY_MAP,
                AGE_CUT_DEFAULT_CATEGORY)
        else:
            subcategory_map, category_map, default_category = (
                AGE_EIGHT_SUBCATEGORY_MAP, AGE_EIGHT_CATEGORY_MAP,
                AGE_EIGHT_DEFAULT_CATEGORY)

        return subcategory_map, category_map, default_category

    def get_count(
            self, index_type=COMMON_COUNT,
            dou_type=ALL_UNIT_TYPES, **kwargs):

        if dou_type != ALL_UNIT_TYPES:
            if ((not self.dou_type and dou_type != GOVERNMENT) or
                    (self.dou_type and
                     self.dou_type.code not in DOU_TYPE_MAP[dou_type])):
                return 0

        result_list = CapacitiesIndex.get_algorithm(
            index_type)().get_result_set(
                cache=self._cache)

        age_category = kwargs.get('age_category', None)
        subcategory_map, category_map, default_category = self._get_age_map(
            age_category)

        total = 0
        for group in result_list:

            # Если у группы есть возрастная подкатегория,
            # то соотношение проводится по ней
            if group.sub_age_cat:
                if group.sub_age_cat.code in subcategory_map[
                        age_category]:
                    total += group.cntFree if group.cntFree >= 0 else 0
            # в противном случае по возрастной категории,
            # которая обязательна.
            elif group.age_cat:
                if group.age_cat.code in category_map[age_category]:
                    total += group.cntFree if group.cntFree >= 0 else 0
            else:
                if age_category and age_category == default_category:
                    total += group.cntFree if group.cntFree >= 0 else 0
        return total


class GisdoGroupHelper(GroupHelper):
    @staticmethod
    def _get_group_cnt_info(unit_scope_ids):
        """
        @type   unit_scope_ids: list
        @param  unit_scope_ids: список садов, для групп которых необходимо
        посчитать кол-во зачислений, кол-во новых направлений и кол-во всех
        направлений

        @return: {dict}
        {15360: {'cnt_direct': 0,
                 'cnt_direct_new': 0,
                 'cnt_pupil': 27},
         15361: {'cnt_direct': 2,
                 'cnt_direct_new': 2,
                 'cnt_pupil': 24},
         ...}
         где 15360 - id группы

        """
        # TODO: Переименовать
        group_cnt_info = {}

        # Подсчет детей в группах
        tmp_query = Pupil.objects.filter(
            temporary_deduct=False,
            grup__unit__id__in=unit_scope_ids
        ).values(
            'grup_id'
        ).annotate(
            cnt=Count('children')
        )
        for i in tmp_query:
            key = i['grup_id']
            group_cnt_info[key] = dict(
                cnt_pupil=i['cnt'],
                cnt_direct=0,
                cnt_direct_new=0,
            )

        # Подсчет новых направлений
        tmp_query = (
            Direct.objects.filter(
                group__unit__id__in=unit_scope_ids,
                status__code='new')
            .values('group_id').annotate(cnt=Count('declaration')))

        for i in tmp_query:
            key = i['group_id']
            if key not in group_cnt_info:
                group_cnt_info[key] = dict(cnt_pupil=0)
            group_cnt_info[key]['cnt_direct_new'] = i['cnt']

        # Подсчет всех направлений
        tmp_query = (
            Direct.objects.filter(
                group__unit__id__in=unit_scope_ids)
            .exclude(status__code__in=DRS.status_alloc())
            .values('group_id').annotate(cnt=Count('declaration')))

        for i in tmp_query:
            key = i['group_id']
            if key not in group_cnt_info:
                group_cnt_info[key] = dict(
                    cnt_pupil=0,
                    cnt_direct_new=0,
                )
            group_cnt_info[key]['cnt_direct'] = i['cnt']

        # Подсчет направлений в группы,
        # где ребенок является времено зачисленным
        # Будем считать, что место свободно.
        tmp_query = (
            Direct.objects.filter(
                group__unit__id__in=unit_scope_ids)
            .exclude(status__code__in=DRS.status_alloc()))

        tmp_query = tmp_query.filter(
            declaration__children__pupil__grup=F('group'),
            declaration__children__pupil__temporary_deduct=False)
        tmp_query = tmp_query.values('group_id').annotate(
            cnt=Count('declaration'))
        for i in tmp_query:
            key = i['group_id']
            if key in group_cnt_info:
                group_cnt_info[key]['cnt_pupil'] = (
                    group_cnt_info[key]['cnt_pupil']-i['cnt'])

        return group_cnt_info
